/* This file is part of the KDE libraries
    Copyright (C) 2009 Norbert Frese <nf2.email@gmail.com>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
 */

#include "giostorage.h"
#include <kdebug.h>
#include <QtCore/QPointer>

GioStorage::GioStorage()
{
    m_isValid = true;
}


GioStorage::~GioStorage()
{
}

bool GioStorage::isValid() const
{
    return m_isValid;
}


void GioStorage::setup(QObject *doneSignalTarget, const char *doneSlot)
{
    emit setupDone(KStorage::lookup(id()), KStorage::UnauthorizedOperation, "setup not supported");
}

void GioStorage::_slotSetupDone(const KStorage &storage, const KStorage::ErrorType error, const QString &errorMessage)
{
    kDebug() << "setup done" << endl;

    emit setupDone(storage, error, errorMessage);
}

////// unmount //////


void GioStorage::tearDown(QObject *doneSignalTarget, const char *doneSlot)
{
    emit teardownDone(KStorage::lookup(id()), KStorage::UnauthorizedOperation, "setup not supported");
}

void GioStorage::_slotTeardownDone(const KStorage &storage, const KStorage::ErrorType error, const QString &errorMessage)
{
    kDebug() << "teardown done" << endl;

    emit teardownDone(storage, error, errorMessage);

}

///// eject /////


void GioStorage::eject(QObject *doneSignalTarget, const char *doneSlot)
{
    emit ejectDone(KStorage::lookup(id()), KStorage::UnauthorizedOperation, "setup not supported");

}

void GioStorage::_slotEjectDone(const KStorage &storage, const KStorage::ErrorType error, const QString &errorMessage)
{
    kDebug() << "eject done" << endl;

    emit ejectDone(storage, error, errorMessage);
}

///// emit signals /////

void GioStorage::_slotChanged()
{
    kDebug() << "changed: " << id();
    emit changed();
}

void GioStorage::_slotRemoved()
{
    kDebug() << "removed: " << id();
    m_isValid = false;
    emit removed();
}

