/* This file is part of the KDE libraries
    Copyright (C) 2009 Norbert Frese <nf2.email@gmail.com>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
 */

#ifndef GIOSTORAGE_H_
#define GIOSTORAGE_H_

#include <glib.h>
#include <gio/gio.h>

#include "kstorageiface.h"
#include "kstorage.h"


class GioStorage : public KStorageIface
{
    Q_OBJECT

public:

    GioStorage();

    ~GioStorage();

    /////

    virtual bool isValid() const;

    virtual QString id() const = 0;

    virtual QString displayName() const = 0;

    virtual QString icon() const = 0;

    virtual QStringList emblems() const = 0;

    virtual QUrl url() const = 0;

    virtual QString mountPointPath() const = 0;

    virtual bool isRemovable() const = 0;

    virtual bool isReleasable() const = 0;

    virtual bool isHotpluggable() const = 0;

    virtual bool isAccessible() const = 0;

    ///// actions ////

    virtual void setup(QObject * doneSignalTarget, const char * doneQSlot);

    /**
     * QSLOT(done(const KStorage &storage, const KStorage::ErrorType error, const QString &errorMessage))
     */
    virtual void tearDown(QObject * doneSignalTarget, const char * doneQSlot);

    /**
     * QSLOT(done(const KStorage &storage, const KStorage::ErrorType error, const QString &errorMessage))
     */
    virtual void eject(QObject * doneSignalTarget, const char * doneQSlot);

    //// members ////

    bool m_isValid;

    ///// signals ////
Q_SIGNALS:

    void setupDone(const KStorage &storage, const KStorage::ErrorType error, const QString &errorMessage);

    void teardownDone(const KStorage &storage, const KStorage::ErrorType error, const QString &errorMessage);

    void ejectDone(const KStorage &storage, const KStorage::ErrorType error, const QString &errorMessage);

    void changed();

    void removed();

public:
    void _slotSetupDone(const KStorage &storage, const KStorage::ErrorType error, const QString &errorMessage);

    void _slotTeardownDone(const KStorage &storage, const KStorage::ErrorType error, const QString &errorMessage);

    void _slotEjectDone(const KStorage &storage, const KStorage::ErrorType error, const QString &errorMessage);


    void _slotChanged();

    void _slotRemoved();

};

#endif /* GIOSTORAGE_H_ */
